@@ -40,14 +40,17 @@ AutoTypePlatformX11::AutoTypePlatformX11()
     m_atomUtf8String = XInternAtom(m_dpy, "UTF8_STRING", True);
     m_atomNetActiveWindow = XInternAtom(m_dpy, "_NET_ACTIVE_WINDOW", True);
 
-    m_classBlacklist << "desktop_window" << "gnome-panel"; // Gnome
-    m_classBlacklist << "kdesktop" << "kicker"; // KDE 3
+    m_classBlacklist << "desktop_window"
+                     << "gnome-panel"; // Gnome
+    m_classBlacklist << "kdesktop"
+                     << "kicker"; // KDE 3
     m_classBlacklist << "Plasma"; // KDE 4
     m_classBlacklist << "plasmashell"; // KDE 5
-    m_classBlacklist << "xfdesktop" << "xfce4-panel"; // Xfce 4
+    m_classBlacklist << "xfdesktop"
+                     << "xfce4-panel"; // Xfce 4
 
     m_currentGlobalKey = static_cast<Qt::Key>(0);
-    m_currentGlobalModifiers = 0;
+    m_currentGlobalModifiers = nullptr;
 
     m_keysymTable = nullptr;
     m_xkb = nullptr;
@@ -146,12 +149,9 @@ bool AutoTypePlatformX11::registerGlobalShortcut(Qt::Key key, Qt::KeyboardModifi
 
     startCatchXErrors();
     XGrabKey(m_dpy, keycode, nativeModifiers, m_rootWindow, True, GrabModeAsync, GrabModeAsync);
-    XGrabKey(m_dpy, keycode, nativeModifiers | Mod2Mask, m_rootWindow, True, GrabModeAsync,
-             GrabModeAsync);
-    XGrabKey(m_dpy, keycode, nativeModifiers | LockMask, m_rootWindow, True, GrabModeAsync,
-             GrabModeAsync);
-    XGrabKey(m_dpy, keycode, nativeModifiers | Mod2Mask | LockMask, m_rootWindow, True,
-             GrabModeAsync, GrabModeAsync);
+    XGrabKey(m_dpy, keycode, nativeModifiers | Mod2Mask, m_rootWindow, True, GrabModeAsync, GrabModeAsync);
+    XGrabKey(m_dpy, keycode, nativeModifiers | LockMask, m_rootWindow, True, GrabModeAsync, GrabModeAsync);
+    XGrabKey(m_dpy, keycode, nativeModifiers | Mod2Mask | LockMask, m_rootWindow, True, GrabModeAsync, GrabModeAsync);
     stopCatchXErrors();
 
     if (!m_xErrorOccurred) {
@@ -160,8 +160,7 @@ bool AutoTypePlatformX11::registerGlobalShortcut(Qt::Key key, Qt::KeyboardModifi
         m_currentGlobalKeycode = keycode;
         m_currentGlobalNativeModifiers = nativeModifiers;
         return true;
-    }
-    else {
+    } else {
         unregisterGlobalShortcut(key, modifiers);
         return false;
     }
@@ -198,7 +197,7 @@ void AutoTypePlatformX11::unregisterGlobalShortcut(Qt::Key key, Qt::KeyboardModi
     XUngrabKey(m_dpy, keycode, nativeModifiers | Mod2Mask | LockMask, m_rootWindow);
 
     m_currentGlobalKey = static_cast<Qt::Key>(0);
-    m_currentGlobalModifiers = 0;
+    m_currentGlobalModifiers = nullptr;
     m_currentGlobalKeycode = 0;
     m_currentGlobalNativeModifiers = 0;
 }
@@ -211,29 +210,25 @@ int AutoTypePlatformX11::platformEventFilter(void* event)
     if (type == XCB_KEY_PRESS || type == XCB_KEY_RELEASE) {
         xcb_key_press_event_t* keyPressEvent = static_cast<xcb_key_press_event_t*>(event);
         if (keyPressEvent->detail == m_currentGlobalKeycode
-                && (keyPressEvent->state & m_modifierMask) == m_currentGlobalNativeModifiers
-                && (!QApplication::activeWindow() || QApplication::activeWindow()->isMinimized())
-                && m_loaded) {
+            && (keyPressEvent->state & m_modifierMask) == m_currentGlobalNativeModifiers
+            && (!QApplication::activeWindow() || QApplication::activeWindow()->isMinimized()) && m_loaded) {
             if (type == XCB_KEY_PRESS) {
                 emit globalShortcutTriggered();
             }
 
             return 1;
         }
-    }
-    else if (type == XCB_MAPPING_NOTIFY) {
+    } else if (type == XCB_MAPPING_NOTIFY) {
         xcb_mapping_notify_event_t* mappingNotifyEvent = static_cast<xcb_mapping_notify_event_t*>(event);
         if (mappingNotifyEvent->request == XCB_MAPPING_KEYBOARD
-                || mappingNotifyEvent->request == XCB_MAPPING_MODIFIER)
-        {
+            || mappingNotifyEvent->request == XCB_MAPPING_MODIFIER) {
             XMappingEvent xMappingEvent;
             memset(&xMappingEvent, 0, sizeof(xMappingEvent));
             xMappingEvent.type = MappingNotify;
             xMappingEvent.display = m_dpy;
             if (mappingNotifyEvent->request == XCB_MAPPING_KEYBOARD) {
                 xMappingEvent.request = MappingKeyboard;
-            }
-            else {
+            } else {
                 xMappingEvent.request = MappingModifier;
             }
             xMappingEvent.first_keycode = mappingNotifyEvent->first_keycode;
@@ -263,13 +258,12 @@ QString AutoTypePlatformX11::windowTitle(Window window, bool useBlacklist)
 
     // the window manager spec says we should read _NET_WM_NAME first, then fall back to WM_NAME
 
-    int retVal = XGetWindowProperty(m_dpy, window, m_atomNetWmName, 0, 1000, False, m_atomUtf8String,
-                                    &type, &format, &nitems, &after, &data);
+    int retVal = XGetWindowProperty(
+        m_dpy, window, m_atomNetWmName, 0, 1000, False, m_atomUtf8String, &type, &format, &nitems, &after, &data);
 
     if ((retVal == 0) && data) {
         title = QString::fromUtf8(reinterpret_cast<char*>(data));
-    }
-    else {
+    } else {
         XTextProperty textProp;
         retVal = XGetTextProperty(m_dpy, window, &textProp, m_atomWmName);
         if ((retVal != 0) && textProp.value) {
@@ -278,12 +272,10 @@ QString AutoTypePlatformX11::windowTitle(Window window, bool useBlacklist)
 
             if (textProp.encoding == m_atomUtf8String) {
                 title = QString::fromUtf8(reinterpret_cast<char*>(textProp.value));
-            }
-            else if ((XmbTextPropertyToTextList(m_dpy, &textProp, &textList, &count) == 0)
-                     && textList && (count > 0)) {
+            } else if ((XmbTextPropertyToTextList(m_dpy, &textProp, &textList, &count) == 0) && textList
+                       && (count > 0)) {
                 title = QString::fromLocal8Bit(textList[0]);
-            }
-            else if (textProp.encoding == m_atomString) {
+            } else if (textProp.encoding == m_atomString) {
                 title = QString::fromLocal8Bit(reinterpret_cast<char*>(textProp.value));
             }
 
@@ -386,8 +378,8 @@ bool AutoTypePlatformX11::isTopLevelWindow(Window window)
     unsigned long nitems;
     unsigned long after;
     unsigned char* data = Q_NULLPTR;
-    int retVal = XGetWindowProperty(m_dpy, window, m_atomWmState, 0, 2, False, m_atomWmState, &type, &format,
-                                    &nitems, &after, &data);
+    int retVal = XGetWindowProperty(
+        m_dpy, window, m_atomWmState, 0, 2, False, m_atomWmState, &type, &format, &nitems, &after, &data);
 
     bool result = false;
 
@@ -408,15 +400,12 @@ KeySym AutoTypePlatformX11::charToKeySym(const QChar& ch)
     ushort unicode = ch.unicode();
 
     /* first check for Latin-1 characters (1:1 mapping) */
-    if ((unicode >= 0x0020 && unicode <= 0x007e)
-            || (unicode >= 0x00a0 && unicode <= 0x00ff)) {
+    if ((unicode >= 0x0020 && unicode <= 0x007e) || (unicode >= 0x00a0 && unicode <= 0x00ff)) {
         return unicode;
     }
 
     /* mapping table generated from keysymdef.h */
-    const uint* match = Tools::binaryFind(m_unicodeToKeysymKeys,
-                                          m_unicodeToKeysymKeys + m_unicodeToKeysymLen,
-                                          unicode);
+    const uint* match = Tools::binaryFind(m_unicodeToKeysymKeys, m_unicodeToKeysymKeys + m_unicodeToKeysymLen, unicode);
     int index = match - m_unicodeToKeysymKeys;
     if (index != m_unicodeToKeysymLen) {
         return m_unicodeToKeysymValues[index];
@@ -483,8 +472,7 @@ KeySym AutoTypePlatformX11::keyToKeySym(Qt::Key key)
     default:
         if (key >= Qt::Key_F1 && key <= Qt::Key_F16) {
             return XK_F1 + (key - Qt::Key_F1);
-        }
-        else {
+        } else {
             return NoSymbol;
         }
     }
@@ -493,13 +481,13 @@ KeySym AutoTypePlatformX11::keyToKeySym(Qt::Key key)
 /*
  * Update the keyboard and modifier mapping.
  * We need the KeyboardMapping for AddKeysym.
- * Modifier mapping is required for clearing the modifiers. 
+ * Modifier mapping is required for clearing the modifiers.
  */
 void AutoTypePlatformX11::updateKeymap()
 {
     int keycode, inx;
     int mod_index, mod_key;
-    XModifierKeymap *modifiers;
+    XModifierKeymap* modifiers;
 
     if (m_xkb) {
         XkbFreeKeyboard(m_xkb, XkbAllComponentsMask, True);
@@ -507,10 +495,9 @@ void AutoTypePlatformX11::updateKeymap()
     m_xkb = getKeyboard();
 
     XDisplayKeycodes(m_dpy, &m_minKeycode, &m_maxKeycode);
-    if (m_keysymTable != NULL) XFree(m_keysymTable);
-    m_keysymTable = XGetKeyboardMapping(m_dpy,
-            m_minKeycode, m_maxKeycode - m_minKeycode + 1,
-            &m_keysymPerKeycode);
+    if (m_keysymTable != nullptr)
+        XFree(m_keysymTable);
+    m_keysymTable = XGetKeyboardMapping(m_dpy, m_minKeycode, m_maxKeycode - m_minKeycode + 1, &m_keysymPerKeycode);
 
     /* determine the keycode to use for remapped keys */
     inx = (m_remapKeycode - m_minKeycode) * m_keysymPerKeycode;
@@ -518,16 +505,16 @@ void AutoTypePlatformX11::updateKeymap()
         for (keycode = m_minKeycode; keycode <= m_maxKeycode; keycode++) {
             inx = (keycode - m_minKeycode) * m_keysymPerKeycode;
             if (m_keysymTable[inx] == NoSymbol) {
-               m_remapKeycode = keycode;
-               m_currentRemapKeysym = NoSymbol;
-               break;
+                m_remapKeycode = keycode;
+                m_currentRemapKeysym = NoSymbol;
+                break;
             }
         }
     }
 
     /* determine the keycode to use for modifiers */
     modifiers = XGetModifierMapping(m_dpy);
-    for (mod_index = ShiftMapIndex; mod_index <= Mod5MapIndex; mod_index ++) {
+    for (mod_index = ShiftMapIndex; mod_index <= Mod5MapIndex; mod_index++) {
         m_modifier_keycode[mod_index] = 0;
         for (mod_key = 0; mod_key < modifiers->max_keypermod; mod_key++) {
             keycode = modifiers->modifiermap[mod_index * modifiers->max_keypermod + mod_key];
@@ -625,7 +612,7 @@ int AutoTypePlatformX11::AddKeysym(KeySym keysym)
         return 0;
     }
 
-    int inx = (m_remapKeycode- m_minKeycode) * m_keysymPerKeycode;
+    int inx = (m_remapKeycode - m_minKeycode) * m_keysymPerKeycode;
     m_keysymTable[inx] = keysym;
     m_currentRemapKeysym = keysym;
 
@@ -644,7 +631,7 @@ int AutoTypePlatformX11::AddKeysym(KeySym keysym)
 void AutoTypePlatformX11::SendKeyEvent(unsigned keycode, bool press)
 {
     XSync(m_dpy, False);
-    int (*oldHandler) (Display*, XErrorEvent*) = XSetErrorHandler(MyErrorHandler);
+    int (*oldHandler)(Display*, XErrorEvent*) = XSetErrorHandler(MyErrorHandler);
 
     XTestFakeKeyEvent(m_dpy, keycode, press, 0);
     XFlush(m_dpy);
@@ -659,7 +646,7 @@ void AutoTypePlatformX11::SendKeyEvent(unsigned keycode, bool press)
 void AutoTypePlatformX11::SendModifiers(unsigned int mask, bool press)
 {
     int mod_index;
-    for (mod_index = ShiftMapIndex; mod_index <= Mod5MapIndex; mod_index ++) {
+    for (mod_index = ShiftMapIndex; mod_index <= Mod5MapIndex; mod_index++) {
         if (mask & (1 << mod_index)) {
             SendKeyEvent(m_modifier_keycode[mod_index], press);
         }
@@ -670,7 +657,7 @@ void AutoTypePlatformX11::SendModifiers(unsigned int mask, bool press)
  * Determines the keycode and modifier mask for the given
  * keysym.
  */
-int AutoTypePlatformX11::GetKeycode(KeySym keysym, unsigned int *mask)
+int AutoTypePlatformX11::GetKeycode(KeySym keysym, unsigned int* mask)
 {
     int keycode = XKeysymToKeycode(m_dpy, keysym);
 
@@ -688,15 +675,15 @@ int AutoTypePlatformX11::GetKeycode(KeySym keysym, unsigned int *mask)
     return 0;
 }
 
-bool AutoTypePlatformX11::keysymModifiers(KeySym keysym, int keycode, unsigned int *mask)
+bool AutoTypePlatformX11::keysymModifiers(KeySym keysym, int keycode, unsigned int* mask)
 {
     int shift, mod;
     unsigned int mods_rtrn;
 
     /* determine whether there is a combination of the modifiers
        (Mod1-Mod5) with or without shift which returns keysym */
-    for (shift = 0; shift < 2; shift ++) {
-        for (mod = ControlMapIndex; mod <= Mod5MapIndex; mod ++) {
+    for (shift = 0; shift < 2; shift++) {
+        for (mod = ControlMapIndex; mod <= Mod5MapIndex; mod++) {
             KeySym keysym_rtrn;
             *mask = (mod == ControlMapIndex) ? shift : shift | (1 << mod);
             XkbTranslateKeyCode(m_xkb, keycode, *mask, &mods_rtrn, &keysym_rtrn);
@@ -709,8 +696,6 @@ bool AutoTypePlatformX11::keysymModifiers(KeySym keysym, int keycode, unsigned i
     return false;
 }
 
-
-
 /*
  * Send sequence of KeyPressed/KeyReleased events to the focused
  * window to simulate keyboard.  If modifiers (shift, control, etc)
@@ -753,7 +738,7 @@ void AutoTypePlatformX11::SendKey(KeySym keysym, unsigned int modifiers)
     if (!modifiers) {
         // check every release_check_mask individually if it affects the keysym we would generate
         // if it doesn't we probably don't need to release it
-        for (int mod_index = ShiftMapIndex; mod_index <= Mod5MapIndex; mod_index ++) {
+        for (int mod_index = ShiftMapIndex; mod_index <= Mod5MapIndex; mod_index++) {
             if (release_check_mask & (1 << mod_index)) {
                 unsigned int mods_rtrn;
                 KeySym keysym_rtrn;
@@ -768,7 +753,8 @@ void AutoTypePlatformX11::SendKey(KeySym keysym, unsigned int modifiers)
         // finally check if the combination of pressed modifiers that we chose to ignore affects the keysym
         unsigned int mods_rtrn;
         KeySym keysym_rtrn;
-        XkbTranslateKeyCode(m_xkb, keycode, wanted_mask | (release_check_mask & ~release_mask), &mods_rtrn, &keysym_rtrn);
+        XkbTranslateKeyCode(
+            m_xkb, keycode, wanted_mask | (release_check_mask & ~release_mask), &mods_rtrn, &keysym_rtrn);
         if (keysym_rtrn != keysym) {
             // oh well, release all the modifiers we don't want
             release_mask = release_check_mask;
@@ -810,7 +796,6 @@ int AutoTypePlatformX11::MyErrorHandler(Display* my_dpy, XErrorEvent* event)
     return 0;
 }
 
-
 AutoTypeExecutorX11::AutoTypeExecutorX11(AutoTypePlatformX11* platform)
     : m_platform(platform)
 {
@@ -864,15 +849,12 @@ bool AutoTypePlatformX11::raiseWindow(WId window)
     QWidget* activeWindow = QApplication::activeWindow();
     if (activeWindow) {
         event.xclient.data.l[2] = activeWindow->internalWinId();
-    }
-    else {
+    } else {
         event.xclient.data.l[2] = 0;
     }
     event.xclient.data.l[3] = 0;
     event.xclient.data.l[4] = 0;
-    XSendEvent(m_dpy, m_rootWindow, False,
-               SubstructureRedirectMask | SubstructureNotifyMask,
-               &event);
+    XSendEvent(m_dpy, m_rootWindow, False, SubstructureRedirectMask | SubstructureNotifyMask, &event);
     XFlush(m_dpy);
 
     return true;
