@@ -1,28 +1,35 @@
 /*
-*  Copyright (C) 2013 Francois Ferrand
-*  Copyright (C) 2017 Sami Vänttinen <sami.vanttinen@protonmail.com>
-*  Copyright (C) 2017 KeePassXC Team <team@keepassxc.org>
-*
-*  This program is free software: you can redistribute it and/or modify
-*  it under the terms of the GNU General Public License as published by
-*  the Free Software Foundation, either version 3 of the License, or
-*  (at your option) any later version.
-*
-*  This program is distributed in the hope that it will be useful,
-*  but WITHOUT ANY WARRANTY; without even the implied warranty of
-*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-*  GNU General Public License for more details.
-*
-*  You should have received a copy of the GNU General Public License
-*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ *  Copyright (C) 2013 Francois Ferrand
+ *  Copyright (C) 2017 Sami Vänttinen <sami.vanttinen@protonmail.com>
+ *  Copyright (C) 2017 KeePassXC Team <team@keepassxc.org>
+ *
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "BrowserSettings.h"
 #include "core/Config.h"
 
-PasswordGenerator BrowserSettings::m_passwordGenerator;
-PassphraseGenerator BrowserSettings::m_passPhraseGenerator;
-HostInstaller BrowserSettings::m_hostInstaller;
+BrowserSettings* BrowserSettings::m_instance(nullptr);
+
+BrowserSettings* BrowserSettings::instance()
+{
+    if (!m_instance) {
+        m_instance = new BrowserSettings();
+    }
+
+    return m_instance;
+}
 
 bool BrowserSettings::isEnabled()
 {
@@ -114,6 +121,16 @@ void BrowserSettings::setAlwaysAllowUpdate(bool alwaysAllowUpdate)
     config()->set("Browser/AlwaysAllowUpdate", alwaysAllowUpdate);
 }
 
+bool BrowserSettings::httpAuthPermission()
+{
+    return config()->get("Browser/HttpAuthPermission", false).toBool();
+}
+
+void BrowserSettings::setHttpAuthPermission(bool httpAuthPermission)
+{
+    config()->set("Browser/HttpAuthPermission", httpAuthPermission);
+}
+
 bool BrowserSettings::searchInAllDatabases()
 {
     return config()->get("Browser/SearchInAllDatabases", false).toBool();
@@ -162,14 +179,14 @@ QString BrowserSettings::customProxyLocation()
     return config()->get("Browser/CustomProxyLocation", "").toString();
 }
 
-void BrowserSettings::setCustomProxyLocation(QString location)
+void BrowserSettings::setCustomProxyLocation(const QString& location)
 {
     config()->set("Browser/CustomProxyLocation", location);
 }
 
 bool BrowserSettings::updateBinaryPath()
 {
-    return config()->get("Browser/UpdateBinaryPath", false).toBool();
+    return config()->get("Browser/UpdateBinaryPath", true).toBool();
 }
 
 void BrowserSettings::setUpdateBinaryPath(bool enabled)
@@ -177,36 +194,80 @@ void BrowserSettings::setUpdateBinaryPath(bool enabled)
     config()->set("Browser/UpdateBinaryPath", enabled);
 }
 
-bool BrowserSettings::chromeSupport() {
+bool BrowserSettings::allowExpiredCredentials()
+{
+    return config()->get("Browser/AllowExpiredCredentials", false).toBool();
+}
+
+void BrowserSettings::setAllowExpiredCredentials(bool enabled)
+{
+    config()->set("Browser/AllowExpiredCredentials", enabled);
+}
+
+bool BrowserSettings::chromeSupport()
+{
     return m_hostInstaller.checkIfInstalled(HostInstaller::SupportedBrowsers::CHROME);
 }
 
-void BrowserSettings::setChromeSupport(bool enabled) {
-    m_hostInstaller.installBrowser(HostInstaller::SupportedBrowsers::CHROME, enabled, supportBrowserProxy(), customProxyLocation());
+void BrowserSettings::setChromeSupport(bool enabled)
+{
+    m_hostInstaller.installBrowser(
+        HostInstaller::SupportedBrowsers::CHROME, enabled, supportBrowserProxy(), customProxyLocation());
 }
 
-bool BrowserSettings::chromiumSupport() {
+bool BrowserSettings::chromiumSupport()
+{
     return m_hostInstaller.checkIfInstalled(HostInstaller::SupportedBrowsers::CHROMIUM);
 }
 
-void BrowserSettings::setChromiumSupport(bool enabled) {
-    m_hostInstaller.installBrowser(HostInstaller::SupportedBrowsers::CHROMIUM, enabled, supportBrowserProxy(), customProxyLocation());
+void BrowserSettings::setChromiumSupport(bool enabled)
+{
+    m_hostInstaller.installBrowser(
+        HostInstaller::SupportedBrowsers::CHROMIUM, enabled, supportBrowserProxy(), customProxyLocation());
 }
 
-bool BrowserSettings::firefoxSupport() {
+bool BrowserSettings::firefoxSupport()
+{
     return m_hostInstaller.checkIfInstalled(HostInstaller::SupportedBrowsers::FIREFOX);
 }
 
-void BrowserSettings::setFirefoxSupport(bool enabled) {
-    m_hostInstaller.installBrowser(HostInstaller::SupportedBrowsers::FIREFOX, enabled, supportBrowserProxy(), customProxyLocation());
+void BrowserSettings::setFirefoxSupport(bool enabled)
+{
+    m_hostInstaller.installBrowser(
+        HostInstaller::SupportedBrowsers::FIREFOX, enabled, supportBrowserProxy(), customProxyLocation());
 }
 
-bool BrowserSettings::vivaldiSupport() {
+bool BrowserSettings::vivaldiSupport()
+{
     return m_hostInstaller.checkIfInstalled(HostInstaller::SupportedBrowsers::VIVALDI);
 }
 
-void BrowserSettings::setVivaldiSupport(bool enabled) {
-    m_hostInstaller.installBrowser(HostInstaller::SupportedBrowsers::VIVALDI, enabled, supportBrowserProxy(), customProxyLocation());
+void BrowserSettings::setVivaldiSupport(bool enabled)
+{
+    m_hostInstaller.installBrowser(
+        HostInstaller::SupportedBrowsers::VIVALDI, enabled, supportBrowserProxy(), customProxyLocation());
+}
+
+bool BrowserSettings::braveSupport()
+{
+  return m_hostInstaller.checkIfInstalled(HostInstaller::SupportedBrowsers::BRAVE);
+}
+
+void BrowserSettings::setBraveSupport(bool enabled)
+{
+  m_hostInstaller.installBrowser(
+                                 HostInstaller::SupportedBrowsers::BRAVE, enabled, supportBrowserProxy(), customProxyLocation());
+}
+
+bool BrowserSettings::torBrowserSupport()
+{
+    return m_hostInstaller.checkIfInstalled(HostInstaller::SupportedBrowsers::TOR_BROWSER);
+}
+
+void BrowserSettings::setTorBrowserSupport(bool enabled)
+{
+    m_hostInstaller.installBrowser(
+        HostInstaller::SupportedBrowsers::TOR_BROWSER, enabled, supportBrowserProxy(), customProxyLocation());
 }
 
 bool BrowserSettings::passwordUseNumbers()
@@ -249,6 +310,66 @@ void BrowserSettings::setPasswordUseSpecial(bool useSpecial)
     config()->set("generator/SpecialChars", useSpecial);
 }
 
+bool BrowserSettings::passwordUseBraces()
+{
+    return config()->get("generator/Braces", PasswordGenerator::DefaultBraces).toBool();
+}
+
+void BrowserSettings::setPasswordUseBraces(bool useBraces)
+{
+    config()->set("generator/Braces", useBraces);
+}
+
+bool BrowserSettings::passwordUsePunctuation()
+{
+    return config()->get("generator/Punctuation", PasswordGenerator::DefaultQuotes).toBool();
+}
+
+void BrowserSettings::setPasswordUsePunctuation(bool usePunctuation)
+{
+    config()->set("generator/Punctuation", usePunctuation);
+}
+
+bool BrowserSettings::passwordUseQuotes()
+{
+    return config()->get("generator/Quotes", PasswordGenerator::DefaultQuotes).toBool();
+}
+
+void BrowserSettings::setPasswordUseQuotes(bool useQuotes)
+{
+    config()->set("generator/Quotes", useQuotes);
+}
+
+bool BrowserSettings::passwordUseDashes()
+{
+    return config()->get("generator/Dashes", PasswordGenerator::DefaultDashes).toBool();
+}
+
+void BrowserSettings::setPasswordUseDashes(bool useDashes)
+{
+    config()->set("generator/Dashes", useDashes);
+}
+
+bool BrowserSettings::passwordUseMath()
+{
+    return config()->get("generator/Math", PasswordGenerator::DefaultMath).toBool();
+}
+
+void BrowserSettings::setPasswordUseMath(bool useMath)
+{
+    config()->set("generator/Math", useMath);
+}
+
+bool BrowserSettings::passwordUseLogograms()
+{
+    return config()->get("generator/Logograms", PasswordGenerator::DefaultLogograms).toBool();
+}
+
+void BrowserSettings::setPasswordUseLogograms(bool useLogograms)
+{
+    config()->set("generator/Logograms", useLogograms);
+}
+
 bool BrowserSettings::passwordUseEASCII()
 {
     return config()->get("generator/EASCII", PasswordGenerator::DefaultEASCII).toBool();
@@ -259,6 +380,26 @@ void BrowserSettings::setPasswordUseEASCII(bool useEASCII)
     config()->set("generator/EASCII", useEASCII);
 }
 
+bool BrowserSettings::advancedMode()
+{
+    return config()->get("generator/AdvancedMode", PasswordGenerator::DefaultAdvancedMode).toBool();
+}
+
+void BrowserSettings::setAdvancedMode(bool advancedMode)
+{
+    config()->set("generator/AdvancedMode", advancedMode);
+}
+
+QString BrowserSettings::passwordExcludedChars()
+{
+    return config()->get("generator/ExcludedChars", PasswordGenerator::DefaultExcludedChars).toString();
+}
+
+void BrowserSettings::setPasswordExcludedChars(const QString& chars)
+{
+    config()->set("generator/ExcludedChars", chars);
+}
+
 int BrowserSettings::passPhraseWordCount()
 {
     return config()->get("generator/WordCount", PassphraseGenerator::DefaultWordCount).toInt();
@@ -274,7 +415,7 @@ QString BrowserSettings::passPhraseWordSeparator()
     return config()->get("generator/WordSeparator", PassphraseGenerator::DefaultSeparator).toString();
 }
 
-void BrowserSettings::setPassPhraseWordSeparator(QString separator)
+void BrowserSettings::setPassPhraseWordSeparator(const QString& separator)
 {
     config()->set("generator/WordSeparator", separator);
 }
@@ -335,6 +476,24 @@ PasswordGenerator::CharClasses BrowserSettings::passwordCharClasses()
     if (passwordUseSpecial()) {
         classes |= PasswordGenerator::SpecialCharacters;
     }
+    if (passwordUseBraces()) {
+        classes |= PasswordGenerator::Braces;
+    }
+    if (passwordUsePunctuation()) {
+        classes |= PasswordGenerator::Punctuation;
+    }
+    if (passwordUseQuotes()) {
+        classes |= PasswordGenerator::Quotes;
+    }
+    if (passwordUseDashes()) {
+        classes |= PasswordGenerator::Dashes;
+    }
+    if (passwordUseMath()) {
+        classes |= PasswordGenerator::Math;
+    }
+    if (passwordUseLogograms()) {
+        classes |= PasswordGenerator::Logograms;
+    }
     if (passwordUseEASCII()) {
         classes |= PasswordGenerator::EASCII;
     }
@@ -368,13 +527,13 @@ QString BrowserSettings::generatePassword()
     }
 }
 
-int BrowserSettings::getbits()
-{
-    return m_passwordGenerator.getbits();
-}
-
-void BrowserSettings::updateBinaryPaths(QString customProxyLocation)
+void BrowserSettings::updateBinaryPaths(const QString& customProxyLocation)
 {
     bool isProxy = supportBrowserProxy();
     m_hostInstaller.updateBinaryPaths(isProxy, customProxyLocation);
 }
+
+bool BrowserSettings::checkIfProxyExists(QString& path)
+{
+    return m_hostInstaller.checkIfProxyExists(supportBrowserProxy(), customProxyLocation(), path);
+}
