@@ -27,8 +27,8 @@ class Remove : public Command
 public:
     Remove();
     ~Remove();
-    int execute(const QStringList& arguments);
-    int removeEntry(Database* database, QString databasePath, QString entryPath);
+    int execute(const QStringList& arguments) override;
+    int removeEntry(Database* database, const QString& databasePath, const QString& entryPath, bool quiet);
 };
 
 #endif // KEEPASSXC_REMOVE_H
