@@ -16,9 +16,12 @@
  */
 
 #include "CustomData.h"
+#include "Clock.h"
 
 #include "core/Global.h"
 
+const QString CustomData::LastModified = "_LAST_MODIFIED";
+
 CustomData::CustomData(QObject* parent)
     : QObject(parent)
 {
@@ -60,6 +63,7 @@ void CustomData::set(const QString& key, const QString& value)
 
     if (addAttribute || changeValue) {
         m_data.insert(key, value);
+        updateLastModified();
         emit customDataModified();
     }
 
@@ -74,6 +78,7 @@ void CustomData::remove(const QString& key)
 
     m_data.remove(key);
 
+    updateLastModified();
     emit removed(key);
     emit customDataModified();
 }
@@ -94,6 +99,7 @@ void CustomData::rename(const QString& oldKey, const QString& newKey)
     m_data.remove(oldKey);
     m_data.insert(newKey, data);
 
+    updateLastModified();
     emit customDataModified();
     emit renamed(oldKey, newKey);
 }
@@ -108,9 +114,19 @@ void CustomData::copyDataFrom(const CustomData* other)
 
     m_data = other->m_data;
 
+    updateLastModified();
     emit reset();
     emit customDataModified();
 }
+
+QDateTime CustomData::getLastModified() const
+{
+    if (m_data.contains(LastModified)) {
+        return Clock::parse(m_data.value(LastModified));
+    }
+    return {};
+}
+
 bool CustomData::operator==(const CustomData& other) const
 {
     return (m_data == other.m_data);
@@ -152,3 +168,13 @@ int CustomData::dataSize() const
     }
     return size;
 }
+
+void CustomData::updateLastModified()
+{
+    if (m_data.size() == 1 && m_data.contains(LastModified)) {
+        m_data.remove(LastModified);
+        return;
+    }
+
+    m_data.insert(LastModified, Clock::currentDateTimeUtc().toString());
+}
