@@ -23,8 +23,8 @@
 #include <QLibrary>
 
 #include "config-keepassx.h"
-#include "core/Global.h"
 #include "core/Config.h"
+#include "core/Global.h"
 
 FilePath* FilePath::m_instance(nullptr);
 
@@ -32,8 +32,7 @@ QString FilePath::dataPath(const QString& name)
 {
     if (name.isEmpty() || name.startsWith('/')) {
         return m_dataPath + name;
-    }
-    else {
+    } else {
         return m_dataPath + "/" + name;
     }
 }
@@ -51,7 +50,7 @@ QString FilePath::pluginPath(const QString& name)
     // for TestAutoType
     pluginPaths << QCoreApplication::applicationDirPath() + "/../src/autotype/test";
 
-#if defined(Q_OS_MAC) && defined(WITH_APP_BUNDLE)
+#if defined(Q_OS_MACOS) && defined(WITH_APP_BUNDLE)
     pluginPaths << QCoreApplication::applicationDirPath() + "/../PlugIns";
 #endif
 
@@ -61,14 +60,12 @@ QString FilePath::pluginPath(const QString& name)
     if (configuredPluginDir != ".") {
         if (QDir(configuredPluginDir).isAbsolute()) {
             pluginPaths << configuredPluginDir;
-        }
-        else {
-            QString relativePluginDir = QString("%1/../%2")
-                    .arg(QCoreApplication::applicationDirPath(), configuredPluginDir);
+        } else {
+            QString relativePluginDir =
+                QString("%1/../%2").arg(QCoreApplication::applicationDirPath(), configuredPluginDir);
             pluginPaths << QDir(relativePluginDir).canonicalPath();
 
-            QString absolutePluginDir = QString("%1/%2")
-                    .arg(KEEPASSX_PREFIX_DIR, configuredPluginDir);
+            QString absolutePluginDir = QString("%1/%2").arg(KEEPASSX_PREFIX_DIR, configuredPluginDir);
             pluginPaths << QDir(absolutePluginDir).canonicalPath();
         }
     }
@@ -151,16 +148,16 @@ QIcon FilePath::icon(const QString& category, const QString& name, bool fromThem
     }
 
     if (icon.isNull()) {
-        const QList<int> pngSizes = { 16, 22, 24, 32, 48, 64, 128 };
+        const QList<int> pngSizes = {16, 22, 24, 32, 48, 64, 128};
         QString filename;
         for (int size : pngSizes) {
-            filename = QString("%1/icons/application/%2x%2/%3.png").arg(m_dataPath, QString::number(size),
-                                                                        combinedName);
+            filename =
+                QString("%1/icons/application/%2x%2/%3.png").arg(m_dataPath, QString::number(size), combinedName);
             if (QFile::exists(filename)) {
                 icon.addFile(filename, QSize(size, size));
             }
         }
-        filename = QString("%1/icons/application/scalable/%2.svgz").arg(m_dataPath, combinedName);
+        filename = QString("%1/icons/application/scalable/%2.svg").arg(m_dataPath, combinedName);
         if (QFile::exists(filename)) {
             icon.addFile(filename);
         }
@@ -189,22 +186,21 @@ QIcon FilePath::onOffIcon(const QString& category, const QString& name)
         if (i == 0) {
             state = QIcon::Off;
             stateName = "off";
-        }
-        else {
+        } else {
             state = QIcon::On;
             stateName = "on";
         }
 
-        const QList<int> pngSizes = { 16, 22, 24, 32, 48, 64, 128 };
+        const QList<int> pngSizes = {16, 22, 24, 32, 48, 64, 128};
         QString filename;
         for (int size : pngSizes) {
-            filename = QString("%1/icons/application/%2x%2/%3-%4.png").arg(m_dataPath, QString::number(size),
-                                                                           combinedName, stateName);
+            filename = QString("%1/icons/application/%2x%2/%3-%4.png")
+                           .arg(m_dataPath, QString::number(size), combinedName, stateName);
             if (QFile::exists(filename)) {
                 icon.addFile(filename, QSize(size, size), QIcon::Normal, state);
             }
         }
-        filename = QString("%1/icons/application/scalable/%2-%3.svgz").arg(m_dataPath, combinedName, stateName);
+        filename = QString("%1/icons/application/scalable/%2-%3.svg").arg(m_dataPath, combinedName, stateName);
         if (QFile::exists(filename)) {
             icon.addFile(filename, QSize(), QIcon::Normal, state);
         }
@@ -227,15 +223,13 @@ FilePath::FilePath()
     else if (testSetDir(QString(KEEPASSX_SOURCE_DIR) + "/share")) {
     }
 #endif
-#if defined(Q_OS_UNIX) && !(defined(Q_OS_MAC) && defined(WITH_APP_BUNDLE))
+#if defined(Q_OS_UNIX) && !(defined(Q_OS_MACOS) && defined(WITH_APP_BUNDLE))
     else if (isDataDirAbsolute && testSetDir(KEEPASSX_DATA_DIR)) {
-    }
-    else if (!isDataDirAbsolute && testSetDir(QString("%1/../%2").arg(appDirPath, KEEPASSX_DATA_DIR))) {
-    }
-    else if (!isDataDirAbsolute && testSetDir(QString("%1/%2").arg(KEEPASSX_PREFIX_DIR, KEEPASSX_DATA_DIR))) {
+    } else if (!isDataDirAbsolute && testSetDir(QString("%1/../%2").arg(appDirPath, KEEPASSX_DATA_DIR))) {
+    } else if (!isDataDirAbsolute && testSetDir(QString("%1/%2").arg(KEEPASSX_PREFIX_DIR, KEEPASSX_DATA_DIR))) {
     }
 #endif
-#if defined(Q_OS_MAC) && defined(WITH_APP_BUNDLE)
+#if defined(Q_OS_MACOS) && defined(WITH_APP_BUNDLE)
     else if (testSetDir(appDirPath + "/../Resources")) {
     }
 #endif
@@ -249,8 +243,7 @@ FilePath::FilePath()
 
     if (m_dataPath.isEmpty()) {
         qWarning("FilePath::DataPath: can't find data dir");
-    }
-    else {
+    } else {
         m_dataPath = QDir::cleanPath(m_dataPath);
     }
 }
@@ -260,8 +253,7 @@ bool FilePath::testSetDir(const QString& dir)
     if (QFile::exists(dir + "/icons/database/C00_Password.png")) {
         m_dataPath = dir;
         return true;
-    }
-    else {
+    } else {
         return false;
     }
 }
