@@ -23,26 +23,30 @@
 #include <QtGlobal>
 
 #if defined(Q_OS_WIN)
-#  if defined(KEEPASSX_BUILDING_CORE)
-#    define KEEPASSX_EXPORT Q_DECL_IMPORT
-#  else
-#    define KEEPASSX_EXPORT Q_DECL_EXPORT
-#  endif
+#if defined(KEEPASSX_BUILDING_CORE)
+#define KEEPASSX_EXPORT Q_DECL_IMPORT
 #else
-#  define KEEPASSX_EXPORT Q_DECL_EXPORT
+#define KEEPASSX_EXPORT Q_DECL_EXPORT
+#endif
+#else
+#define KEEPASSX_EXPORT Q_DECL_EXPORT
 #endif
 
 #ifndef QUINT32_MAX
 #define QUINT32_MAX 4294967295U
 #endif
 
-template <typename T> struct AddConst { typedef const T Type; };
+template <typename T> struct AddConst
+{
+    typedef const T Type;
+};
 
 // this adds const to non-const objects (like std::as_const)
-template <typename T>
-constexpr typename AddConst<T>::Type& asConst(T &t) noexcept { return t; }
+template <typename T> constexpr typename AddConst<T>::Type& asConst(T& t) noexcept
+{
+    return t;
+}
 // prevent rvalue arguments:
-template <typename T>
-void asConst(const T&&) = delete;
+template <typename T> void asConst(const T&&) = delete;
 
 #endif // KEEPASSX_GLOBAL_H
