@@ -21,16 +21,19 @@
 #include "crypto/Random.h"
 #include <zxcvbn.h>
 
+const char* PasswordGenerator::DefaultExcludedChars = "";
+
 PasswordGenerator::PasswordGenerator()
     : m_length(0)
-    , m_classes(0)
-    , m_flags(0)
+    , m_classes(nullptr)
+    , m_flags(nullptr)
+    , m_excluded(PasswordGenerator::DefaultExcludedChars)
 {
 }
 
-double PasswordGenerator::calculateEntropy(QString password)
+double PasswordGenerator::calculateEntropy(const QString& password)
 {
-    return ZxcvbnMatch(password.toLatin1(), 0, 0);
+    return ZxcvbnMatch(password.toLatin1(), nullptr, nullptr);
 }
 
 void PasswordGenerator::setLength(int length)
@@ -45,7 +48,7 @@ void PasswordGenerator::setLength(int length)
 void PasswordGenerator::setCharClasses(const CharClasses& classes)
 {
     if (classes == 0) {
-        m_classes =  DefaultCharset;
+        m_classes = DefaultCharset;
         return;
     }
     m_classes = classes;
@@ -56,6 +59,11 @@ void PasswordGenerator::setFlags(const GeneratorFlags& flags)
     m_flags = flags;
 }
 
+void PasswordGenerator::setExcludedChars(const QString& chars)
+{
+    m_excluded = chars;
+}
+
 QString PasswordGenerator::generatePassword() const
 {
     Q_ASSERT(isValid());
@@ -72,30 +80,29 @@ QString PasswordGenerator::generatePassword() const
     QString password;
 
     if (m_flags & CharFromEveryGroup) {
-        for (int i = 0; i < groups.size(); i++) {
-            int pos = randomGen()->randomUInt(groups[i].size());
+        for (const auto& group : groups) {
+            int pos = randomGen()->randomUInt(static_cast<quint32>(group.size()));
 
-            password.append(groups[i][pos]);
+            password.append(group[pos]);
         }
 
         for (int i = groups.size(); i < m_length; i++) {
-            int pos = randomGen()->randomUInt(passwordChars.size());
+            int pos = randomGen()->randomUInt(static_cast<quint32>(passwordChars.size()));
 
             password.append(passwordChars[pos]);
         }
 
         // shuffle chars
         for (int i = (password.size() - 1); i >= 1; i--) {
-            int j = randomGen()->randomUInt(i + 1);
+            int j = randomGen()->randomUInt(static_cast<quint32>(i + 1));
 
             QChar tmp = password[i];
             password[i] = password[j];
             password[j] = tmp;
         }
-    }
-    else {
+    } else {
         for (int i = 0; i < m_length; i++) {
-            int pos = randomGen()->randomUInt(passwordChars.size());
+            int pos = randomGen()->randomUInt(static_cast<quint32>(passwordChars.size()));
 
             password.append(passwordChars[pos]);
         }
@@ -104,28 +111,11 @@ QString PasswordGenerator::generatePassword() const
     return password;
 }
 
-int PasswordGenerator::getbits() const
-{
-    const QVector<PasswordGroup> groups = passwordGroups();
-
-    int bits = 0;
-    QVector<QChar> passwordChars;
-    for (const PasswordGroup& group: groups) {
-        bits += group.size();
-    }
-
-    bits *= m_length;
-
-    return bits;
-}
-
-
 bool PasswordGenerator::isValid() const
 {
     if (m_classes == 0) {
         return false;
-    }
-    else if (m_length == 0) {
+    } else if (m_length == 0) {
         return false;
     }
 
@@ -133,7 +123,7 @@ bool PasswordGenerator::isValid() const
         return false;
     }
 
-    return true;
+    return !passwordGroups().isEmpty();
 }
 
 QVector<PasswordGroup> PasswordGenerator::passwordGroups() const
@@ -143,7 +133,8 @@ QVector<PasswordGroup> PasswordGenerator::passwordGroups() const
     if (m_classes & LowerLetters) {
         PasswordGroup group;
 
-        for (int i = 97; i < (97 + 26); i++) {
+        for (int i = 97; i <= (97 + 25); i++) {
+
             if ((m_flags & ExcludeLookAlike) && (i == 108)) { // "l"
                 continue;
             }
@@ -156,7 +147,8 @@ QVector<PasswordGroup> PasswordGenerator::passwordGroups() const
     if (m_classes & UpperLetters) {
         PasswordGroup group;
 
-        for (int i = 65; i < (65 + 26); i++) {
+        for (int i = 65; i <= (65 + 25); i++) {
+
             if ((m_flags & ExcludeLookAlike) && (i == 73 || i == 79)) { // "I" and "O"
                 continue;
             }
@@ -179,28 +171,79 @@ QVector<PasswordGroup> PasswordGenerator::passwordGroups() const
 
         passwordGroups.append(group);
     }
-    if (m_classes & SpecialCharacters) {
+    if (m_classes & Braces) {
         PasswordGroup group;
 
-        for (int i = 33; i <= 47; i++) {
-            group.append(i);
+        // ()[]{}
+        group.append(40);
+        group.append(41);
+        group.append(91);
+        group.append(93);
+        group.append(123);
+        group.append(125);
+
+        passwordGroups.append(group);
+    }
+    if (m_classes & Punctuation) {
+        PasswordGroup group;
+
+        // .,:;
+        group.append(44);
+        group.append(46);
+        group.append(58);
+        group.append(59);
+
+        passwordGroups.append(group);
+    }
+    if (m_classes & Quotes) {
+        PasswordGroup group;
+
+        // "'
+        group.append(34);
+        group.append(39);
+
+        passwordGroups.append(group);
+    }
+    if (m_classes & Dashes) {
+        PasswordGroup group;
+
+        // -/\_|
+        group.append(45);
+        group.append(47);
+        group.append(92);
+        group.append(95);
+        if (!(m_flags & ExcludeLookAlike)) {
+            group.append(124); // "|"
         }
 
-        for (int i = 58; i <= 64; i++) {
-            group.append(i);
-        }
+        passwordGroups.append(group);
+    }
+    if (m_classes & Math) {
+        PasswordGroup group;
 
-        for (int i = 91; i <= 96; i++) {
-            group.append(i);
-        }
+        // !*+-<=>?
+        group.append(33);
+        group.append(42);
+        group.append(43);
+        group.append(60);
+        group.append(61);
+        group.append(62);
+        group.append(63);
 
-        for (int i = 123; i <= 126; i++) {
-            if ((m_flags & ExcludeLookAlike) && (i == 124)) { // "|"
-                continue;
-            }
+        passwordGroups.append(group);
+    }
+    if (m_classes & Logograms) {
+        PasswordGroup group;
 
+        // #$%&
+        for (int i = 35; i <= 38; i++) {
             group.append(i);
         }
+        // @^`~
+        group.append(64);
+        group.append(94);
+        group.append(96);
+        group.append(126);
 
         passwordGroups.append(group);
     }
@@ -223,6 +266,27 @@ QVector<PasswordGroup> PasswordGenerator::passwordGroups() const
         passwordGroups.append(group);
     }
 
+    // Loop over character groups and remove excluded characters from them;
+    // remove empty groups
+    int i = 0;
+    while (i != passwordGroups.size()) {
+        PasswordGroup group = passwordGroups[i];
+
+        for (QChar ch : m_excluded) {
+            int j = group.indexOf(ch);
+            while (j != -1) {
+                group.remove(j);
+                j = group.indexOf(ch);
+            }
+        }
+        if (!group.isEmpty()) {
+            passwordGroups.replace(i, group);
+            ++i;
+        } else {
+            passwordGroups.remove(i);
+        }
+    }
+
     return passwordGroups;
 }
 
@@ -239,7 +303,22 @@ int PasswordGenerator::numCharClasses() const
     if (m_classes & Numbers) {
         numClasses++;
     }
-    if (m_classes & SpecialCharacters) {
+    if (m_classes & Braces) {
+        numClasses++;
+    }
+    if (m_classes & Punctuation) {
+        numClasses++;
+    }
+    if (m_classes & Quotes) {
+        numClasses++;
+    }
+    if (m_classes & Dashes) {
+        numClasses++;
+    }
+    if (m_classes & Math) {
+        numClasses++;
+    }
+    if (m_classes & Logograms) {
         numClasses++;
     }
     if (m_classes & EASCII) {
