@@ -20,6 +20,7 @@
 
 #include <QPair>
 #include <QString>
+#include <QLocale>
 
 class Translator
 {
@@ -28,8 +29,8 @@ public:
     static QList<QPair<QString, QString>> availableLanguages();
 
 private:
-    static bool installTranslator(const QString& language, const QString& path);
-    static bool installQtTranslator(const QString& language, const QString& path);
+    static bool installTranslator(const QLocale& locale, const QString& path);
+    static bool installQtTranslator(const QLocale& locale, const QString& path);
 };
 
 #endif // KEEPASSX_TRANSLATOR_H
