@@ -21,7 +21,6 @@
 #include "MainWindow.h"
 #include "core/Config.h"
 
-#include <QAbstractNativeEventFilter>
 #include <QFileInfo>
 #include <QFileOpenEvent>
 #include <QLockFile>
@@ -32,69 +31,35 @@
 #include "autotype/AutoType.h"
 #include "core/Global.h"
 
+#if defined(Q_OS_WIN) || (defined(Q_OS_UNIX) && !defined(Q_OS_MACOS))
+#include "core/OSEventFilter.h"
+#endif
+
 #if defined(Q_OS_UNIX)
 #include <signal.h>
-#include <unistd.h>
 #include <sys/socket.h>
+#include <unistd.h>
 #endif
 
-namespace {
-constexpr int WaitTimeoutMSec = 150;
-const char BlockSizeProperty[] = "blockSize";
-}
-
-#if defined(Q_OS_UNIX) && !defined(Q_OS_OSX)
-class XcbEventFilter : public QAbstractNativeEventFilter
+namespace
 {
-public:
-    bool nativeEventFilter(const QByteArray& eventType, void* message, long* result) override
-    {
-        Q_UNUSED(result)
-
-        if (eventType == QByteArrayLiteral("xcb_generic_event_t")) {
-            int retCode = autoType()->callEventFilter(message);
-            if (retCode == 1) {
-                return true;
-            }
-        }
-
-        return false;
-    }
-};
-#elif defined(Q_OS_WIN)
-class WinEventFilter : public QAbstractNativeEventFilter
-{
-public:
-    bool nativeEventFilter(const QByteArray& eventType, void* message, long* result) override
-    {
-        Q_UNUSED(result);
-
-        if (eventType == QByteArrayLiteral("windows_generic_MSG")
-                || eventType == QByteArrayLiteral("windows_dispatcher_MSG")) {
-            int retCode = autoType()->callEventFilter(message);
-            if (retCode == 1) {
-                return true;
-            }
-        }
-
-        return false;
-    }
-};
-#endif
+    constexpr int WaitTimeoutMSec = 150;
+    const char BlockSizeProperty[] = "blockSize";
+} // namespace
 
 Application::Application(int& argc, char** argv)
     : QApplication(argc, argv)
-    , m_mainWindow(nullptr)
 #ifdef Q_OS_UNIX
     , m_unixSignalNotifier(nullptr)
 #endif
     , m_alreadyRunning(false)
     , m_lockFile(nullptr)
+#if defined(Q_OS_WIN) || (defined(Q_OS_UNIX) && !defined(Q_OS_MACOS))
+    , m_osEventFilter(new OSEventFilter())
+{
+    installNativeEventFilter(m_osEventFilter.data());
+#else
 {
-#if defined(Q_OS_UNIX) && !defined(Q_OS_OSX)
-    installNativeEventFilter(new XcbEventFilter());
-#elif defined(Q_OS_WIN)
-    installNativeEventFilter(new WinEventFilter());
 #endif
 #if defined(Q_OS_UNIX)
     registerUnixSignals();
@@ -109,8 +74,8 @@ Application::Application(int& argc, char** argv)
         identifier += "-" + userName;
     }
 #ifdef QT_DEBUG
-        // In DEBUG mode don't interfere with Release instances
-        identifier += "-DEBUG";
+    // In DEBUG mode don't interfere with Release instances
+    identifier += "-DEBUG";
 #endif
     QString lockName = identifier + ".lock";
     m_socketName = identifier + ".socket";
@@ -146,9 +111,9 @@ Application::Application(int& argc, char** argv)
 
             if (!m_alreadyRunning) {
                 // If we get here then the original instance is likely dead
-                qWarning() << QCoreApplication::translate("Main",
-                                "Existing single-instance lock file is invalid. Launching new instance.")
-                                .toUtf8().constData();
+                qWarning() << QObject::tr("Existing single-instance lock file is invalid. Launching new instance.")
+                                  .toUtf8()
+                                  .constData();
 
                 // forceably reset the lock file
                 m_lockFile->removeStaleLockFile();
@@ -160,9 +125,8 @@ Application::Application(int& argc, char** argv)
         break;
     }
     default:
-        qWarning() << QCoreApplication::translate("Main",
-                        "The lock file could not be created. Single-instance mode disabled.")
-                        .toUtf8().constData();
+        qWarning()
+            << QObject::tr("The lock file could not be created. Single-instance mode disabled.").toUtf8().constData();
     }
 }
 
@@ -175,16 +139,6 @@ Application::~Application()
     }
 }
 
-QWidget* Application::mainWindow() const
-{
-    return m_mainWindow;
-}
-
-void Application::setMainWindow(QWidget* mainWindow)
-{
-    m_mainWindow = mainWindow;
-}
-
 bool Application::event(QEvent* event)
 {
     // Handle Apple QFileOpenEvent from finder (double click on .kdbx file)
@@ -192,7 +146,7 @@ bool Application::event(QEvent* event)
         emit openFile(static_cast<QFileOpenEvent*>(event)->file());
         return true;
     }
-#ifdef Q_OS_MAC
+#ifdef Q_OS_MACOS
     // restore main window when clicking on the docker icon
     else if (event->type() == QEvent::ApplicationActivate) {
         emit applicationActivated();
@@ -214,17 +168,17 @@ void Application::registerUnixSignals()
         // application will be unresponsive to signals such as SIGINT or SIGTERM
         return;
     }
-    
-    QVector<int> const handledSignals = { SIGQUIT, SIGINT, SIGTERM, SIGHUP };
-    for (auto s: handledSignals) {
+
+    QVector<int> const handledSignals = {SIGQUIT, SIGINT, SIGTERM, SIGHUP};
+    for (auto s : handledSignals) {
         struct sigaction sigAction;
-        
+
         sigAction.sa_handler = handleUnixSignal;
         sigemptyset(&sigAction.sa_mask);
         sigAction.sa_flags = 0 | SA_RESTART;
         sigaction(s, &sigAction, nullptr);
     }
-    
+
     m_unixSignalNotifier = new QSocketNotifier(unixSignalSocket[1], QSocketNotifier::Read, this);
     connect(m_unixSignalNotifier, SIGNAL(activated(int)), this, SLOT(quitBySignal()));
 }
@@ -232,16 +186,15 @@ void Application::registerUnixSignals()
 void Application::handleUnixSignal(int sig)
 {
     switch (sig) {
-        case SIGQUIT:
-        case SIGINT:
-        case SIGTERM:
-        {
-            char buf = 0;
-            Q_UNUSED(::write(unixSignalSocket[0], &buf, sizeof(buf)));
-            return;
-        }
-        case SIGHUP:
-            return;
+    case SIGQUIT:
+    case SIGINT:
+    case SIGTERM: {
+        char buf = 0;
+        Q_UNUSED(::write(unixSignalSocket[0], &buf, sizeof(buf)));
+        return;
+    }
+    case SIGHUP:
+        return;
     }
 }
 
@@ -289,7 +242,7 @@ void Application::socketReadyRead()
 
     QStringList fileNames;
     in >> fileNames;
-    for (const QString& fileName: asConst(fileNames)) {
+    for (const QString& fileName : asConst(fileNames)) {
         const QFileInfo fInfo(fileName);
         if (fInfo.isFile() && fInfo.suffix().toLower() == "kdbx") {
             emit openFile(fileName);
@@ -319,8 +272,7 @@ bool Application::sendFileNamesToRunningInstance(const QStringList& fileNames)
     QByteArray data;
     QDataStream out(&data, QIODevice::WriteOnly);
     out.setVersion(QDataStream::Qt_5_0);
-    out << quint32(0)
-        << fileNames;
+    out << quint32(0) << fileNames;
     out.device()->seek(0);
     out << quint32(data.size() - sizeof(quint32));
 
@@ -329,4 +281,3 @@ bool Application::sendFileNamesToRunningInstance(const QStringList& fileNames)
     const bool disconnected = client.waitForDisconnected(WaitTimeoutMSec);
     return writeOk && disconnected;
 }
-
