@@ -24,13 +24,28 @@
 #include "core/Group.h"
 #include "gui/EditWidget.h"
 
+class CustomData;
 class EditWidgetIcons;
 class EditWidgetProperties;
 
-namespace Ui {
+namespace Ui
+{
     class EditGroupWidgetMain;
     class EditWidget;
-}
+} // namespace Ui
+
+class IEditGroupPage
+{
+public:
+    virtual ~IEditGroupPage()
+    {
+    }
+    virtual QString name() = 0;
+    virtual QIcon icon() = 0;
+    virtual QWidget* createWidget() = 0;
+    virtual void set(QWidget* widget, Group* tempoaryGroup, QSharedPointer<Database> database) = 0;
+    virtual void assign(QWidget* widget) = 0;
+};
 
 class EditGroupWidget : public EditWidget
 {
@@ -40,9 +55,11 @@ public:
     explicit EditGroupWidget(QWidget* parent = nullptr);
     ~EditGroupWidget();
 
-    void loadGroup(Group* group, bool create, Database* database);
+    void loadGroup(Group* group, bool create, const QSharedPointer<Database>& database);
     void clear();
 
+    void addEditPage(IEditGroupPage* page);
+
 signals:
     void editFinished(bool accepted);
     void messageEditEntry(QString, MessageWidget::MessageType);
@@ -57,13 +74,20 @@ private:
     void addTriStateItems(QComboBox* comboBox, bool inheritValue);
     int indexFromTriState(Group::TriState triState);
     Group::TriState triStateFromIndex(int index);
+    void setupModifiedTracking();
 
     const QScopedPointer<Ui::EditGroupWidgetMain> m_mainUi;
-    QWidget* const m_editGroupWidgetMain;
-    EditWidgetIcons* const m_editGroupWidgetIcons;
-    EditWidgetProperties* const m_editWidgetProperties;
-    Group* m_group;
-    Database* m_database;
+
+    QPointer<QWidget> m_editGroupWidgetMain;
+    QPointer<EditWidgetIcons> m_editGroupWidgetIcons;
+    QPointer<EditWidgetProperties> m_editWidgetProperties;
+
+    QScopedPointer<Group> m_temporaryGroup;
+    QPointer<Group> m_group;
+    QSharedPointer<Database> m_db;
+
+    class ExtraPage;
+    QList<ExtraPage> m_extraPages;
 
     Q_DISABLE_COPY(EditGroupWidget)
 };
