@@ -46,8 +46,8 @@
 #include <QStringBuilder>
 
 #if defined(WITH_XC_KEESHARE_SECURE)
-#include <quazip5/quazip.h>
-#include <quazip5/quazipfile.h>
+#include <quazip.h>
+#include <quazipfile.h>
 #endif
 
 namespace
