@@ -1,33 +1,34 @@
 /*
-*  Copyright (C) 2017 KeePassXC Team <team@keepassxc.org>
-*
-*  This program is free software: you can redistribute it and/or modify
-*  it under the terms of the GNU General Public License as published by
-*  the Free Software Foundation, either version 3 of the License, or
-*  (at your option) any later version.
-*
-*  This program is distributed in the hope that it will be useful,
-*  but WITHOUT ANY WARRANTY; without even the implied warranty of
-*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-*  GNU General Public License for more details.
-*
-*  You should have received a copy of the GNU General Public License
-*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ *  Copyright (C) 2017 KeePassXC Team <team@keepassxc.org>
+ *
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
-#include <QCoreApplication>
 #include "NativeMessagingHost.h"
+#include <QCoreApplication>
 
 #ifdef Q_OS_WIN
-#include <Winsock2.h>
+#include <winsock2.h>
 #endif
 
-NativeMessagingHost::NativeMessagingHost() : NativeMessagingBase(true)
+NativeMessagingHost::NativeMessagingHost()
+    : NativeMessagingBase(true)
 {
     m_localSocket = new QLocalSocket();
     m_localSocket->connectToServer(getLocalServerPath());
     m_localSocket->setReadBufferSize(NATIVE_MSG_MAX_LENGTH);
-  
+
     int socketDesc = m_localSocket->socketDescriptor();
     if (socketDesc) {
         int max = NATIVE_MSG_MAX_LENGTH;
@@ -35,11 +36,13 @@ NativeMessagingHost::NativeMessagingHost() : NativeMessagingBase(true)
     }
 #ifdef Q_OS_WIN
     m_running.store(true);
-    m_future = QtConcurrent::run(this, static_cast<void(NativeMessagingHost::*)()>(&NativeMessagingHost::readNativeMessages));
+    m_future = QtConcurrent::run(this, &NativeMessagingHost::readNativeMessages);
 #endif
     connect(m_localSocket, SIGNAL(readyRead()), this, SLOT(newLocalMessage()));
     connect(m_localSocket, SIGNAL(disconnected()), this, SLOT(deleteSocket()));
-    connect(m_localSocket, SIGNAL(stateChanged(QLocalSocket::LocalSocketState)), this, SLOT(socketStateChanged(QLocalSocket::LocalSocketState)));
+    connect(m_localSocket,
+            SIGNAL(stateChanged(QLocalSocket::LocalSocketState)),
+            SLOT(socketStateChanged(QLocalSocket::LocalSocketState)));
 }
 
 NativeMessagingHost::~NativeMessagingHost()
@@ -97,7 +100,7 @@ bool NativeMessagingHost::readStdIn(const quint32 length)
         m_localSocket->write(arr.constData(), arr.length());
         m_localSocket->flush();
     }
-    
+
     return true;
 }
 
