@@ -19,7 +19,7 @@
 #ifndef AGENTSETTINGSPAGE_H
 #define AGENTSETTINGSPAGE_H
 
-#include "gui/SettingsWidget.h"
+#include "gui/ApplicationSettingsWidget.h"
 #include "gui/DatabaseTabWidget.h"
 
 class AgentSettingsPage : public ISettingsPage
