@@ -51,6 +51,8 @@ private slots:
     void testExtract();
     void testGenerate_data();
     void testGenerate();
+    void testKeyFileOption();
+    void testNoPasswordOption();
     void testList();
     void testLocate();
     void testMerge();
@@ -61,8 +63,12 @@ private slots:
 private:
     QByteArray m_dbData;
     QByteArray m_dbData2;
+    QByteArray m_keyFileProtectedDbData;
+    QByteArray m_keyFileProtectedNoPasswordDbData;
     QScopedPointer<TemporaryFile> m_dbFile;
     QScopedPointer<TemporaryFile> m_dbFile2;
+    QScopedPointer<TemporaryFile> m_keyFileProtectedDbFile;
+    QScopedPointer<TemporaryFile> m_keyFileProtectedNoPasswordDbFile;
     QScopedPointer<TemporaryFile> m_stdoutFile;
     QScopedPointer<TemporaryFile> m_stderrFile;
     QScopedPointer<TemporaryFile> m_stdinFile;
