@@ -15,7 +15,6 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 #ifndef KEEPASSX_TESTENTRYSEARCHER_H
 #define KEEPASSX_TESTENTRYSEARCHER_H
 
@@ -29,15 +28,16 @@ class TestEntrySearcher : public QObject
     Q_OBJECT
 
 private slots:
-    void initTestCase();
-    void cleanupTestCase();
+    void init();
+    void cleanup();
 
     void testAndConcatenationInSearch();
     void testSearch();
     void testAllAttributesAreSearched();
+    void testSearchTermParser();
 
 private:
-    Group* m_groupRoot;
+    Group* m_rootGroup;
     EntrySearcher m_entrySearcher;
     QList<Entry*> m_searchResult;
 };
