@@ -18,6 +18,7 @@
 
 #include "TestGroup.h"
 #include "TestGlobal.h"
+#include "mock/MockClock.h"
 
 #include <QSignalSpy>
 
@@ -26,6 +27,11 @@
 
 QTEST_GUILESS_MAIN(TestGroup)
 
+namespace
+{
+    MockClock* m_clock = nullptr;
+}
+
 void TestGroup::initTestCase()
 {
     qRegisterMetaType<Entry*>("Entry*");
@@ -33,6 +39,19 @@ void TestGroup::initTestCase()
     QVERIFY(Crypto::init());
 }
 
+void TestGroup::init()
+{
+    Q_ASSERT(m_clock == nullptr);
+    m_clock = new MockClock(2010, 5, 5, 10, 30, 10);
+    MockClock::setup(m_clock);
+}
+
+void TestGroup::cleanup()
+{
+    MockClock::teardown();
+    m_clock = nullptr;
+}
+
 void TestGroup::testParenting()
 {
     Database* db = new Database();
@@ -93,7 +112,7 @@ void TestGroup::testParenting()
     g4->setName("test");
     g3->setName("test");
     g1->setName("test");
-    g3->setIcon(Uuid::random());
+    g3->setIcon(QUuid::createUuid());
     g1->setIcon(2);
     QCOMPARE(spy.count(), 6);
     delete db;
@@ -293,12 +312,12 @@ void TestGroup::testCopyCustomIcon()
 {
     QScopedPointer<Database> dbSource(new Database());
 
-    Uuid groupIconUuid = Uuid::random();
+    QUuid groupIconUuid = QUuid::createUuid();
     QImage groupIcon(16, 16, QImage::Format_RGB32);
     groupIcon.setPixel(0, 0, qRgb(255, 0, 0));
     dbSource->metadata()->addCustomIcon(groupIconUuid, groupIcon);
 
-    Uuid entryIconUuid = Uuid::random();
+    QUuid entryIconUuid = QUuid::createUuid();
     QImage entryIcon(16, 16, QImage::Format_RGB32);
     entryIcon.setPixel(0, 0, qRgb(255, 0, 0));
     dbSource->metadata()->addCustomIcon(entryIconUuid, entryIcon);
@@ -389,13 +408,14 @@ void TestGroup::testClone()
     QVERIFY(clonedGroupNewUuid->uuid() != originalGroup->uuid());
 
     // Making sure the new modification date is not the same.
-    QTest::qSleep(1);
+    m_clock->advanceSecond(1);
 
-    QScopedPointer<Group> clonedGroupResetTimeInfo(originalGroup->clone(Entry::CloneNoFlags,
-                                                                        Group::CloneNewUuid | Group::CloneResetTimeInfo));
+    QScopedPointer<Group> clonedGroupResetTimeInfo(
+        originalGroup->clone(Entry::CloneNoFlags, Group::CloneNewUuid | Group::CloneResetTimeInfo));
     QCOMPARE(clonedGroupResetTimeInfo->entries().size(), 0);
     QVERIFY(clonedGroupResetTimeInfo->uuid() != originalGroup->uuid());
-    QVERIFY(clonedGroupResetTimeInfo->timeInfo().lastModificationTime() != originalGroup->timeInfo().lastModificationTime());
+    QVERIFY(clonedGroupResetTimeInfo->timeInfo().lastModificationTime()
+            != originalGroup->timeInfo().lastModificationTime());
 }
 
 void TestGroup::testCopyCustomIcons()
@@ -411,25 +431,25 @@ void TestGroup::testCopyCustomIcons()
 
     QScopedPointer<Group> group1(new Group());
     group1->setParent(dbSource->rootGroup());
-    Uuid group1Icon = Uuid::random();
+    QUuid group1Icon = QUuid::createUuid();
     dbSource->metadata()->addCustomIcon(group1Icon, iconImage1);
     group1->setIcon(group1Icon);
 
     QScopedPointer<Group> group2(new Group());
     group2->setParent(group1.data());
-    Uuid group2Icon = Uuid::random();
+    QUuid group2Icon = QUuid::createUuid();
     dbSource->metadata()->addCustomIcon(group2Icon, iconImage1);
     group2->setIcon(group2Icon);
 
     QScopedPointer<Entry> entry1(new Entry());
     entry1->setGroup(group2.data());
-    Uuid entry1IconOld = Uuid::random();
+    QUuid entry1IconOld = QUuid::createUuid();
     dbSource->metadata()->addCustomIcon(entry1IconOld, iconImage1);
     entry1->setIcon(entry1IconOld);
 
     // add history item
     entry1->beginUpdate();
-    Uuid entry1IconNew = Uuid::random();
+    QUuid entry1IconNew = QUuid::createUuid();
     dbSource->metadata()->addCustomIcon(entry1IconNew, iconImage1);
     entry1->setIcon(entry1IconNew);
     entry1->endUpdate();
@@ -458,7 +478,7 @@ void TestGroup::testFindEntry()
     Entry* entry1 = new Entry();
     entry1->setTitle(QString("entry1"));
     entry1->setGroup(db->rootGroup());
-    entry1->setUuid(Uuid::random());
+    entry1->setUuid(QUuid::createUuid());
 
     Group* group1 = new Group();
     group1->setName("group1");
@@ -467,63 +487,70 @@ void TestGroup::testFindEntry()
 
     entry2->setTitle(QString("entry2"));
     entry2->setGroup(group1);
-    entry2->setUuid(Uuid::random());
+    entry2->setUuid(QUuid::createUuid());
 
     group1->setParent(db->rootGroup());
 
     Entry* entry;
 
-    entry = db->rootGroup()->findEntry(entry1->uuid().toHex());
-    QVERIFY(entry != nullptr);
+    entry = db->rootGroup()->findEntryByUuid(entry1->uuid());
+    QVERIFY(entry);
     QCOMPARE(entry->title(), QString("entry1"));
 
-    entry = db->rootGroup()->findEntry(QString("entry1"));
-    QVERIFY(entry != nullptr);
+    entry = db->rootGroup()->findEntryByPath(QString("entry1"));
+    QVERIFY(entry);
     QCOMPARE(entry->title(), QString("entry1"));
 
     // We also can find the entry with the leading slash.
-    entry = db->rootGroup()->findEntry(QString("/entry1"));
-    QVERIFY(entry != nullptr);
+    entry = db->rootGroup()->findEntryByPath(QString("/entry1"));
+    QVERIFY(entry);
     QCOMPARE(entry->title(), QString("entry1"));
 
     // But two slashes should not be accepted.
-    entry = db->rootGroup()->findEntry(QString("//entry1"));
-    QVERIFY(entry == nullptr);
+    entry = db->rootGroup()->findEntryByPath(QString("//entry1"));
+    QVERIFY(!entry);
 
-    entry = db->rootGroup()->findEntry(entry2->uuid().toHex());
-    QVERIFY(entry != nullptr);
+    entry = db->rootGroup()->findEntryByUuid(entry2->uuid());
+    QVERIFY(entry);
     QCOMPARE(entry->title(), QString("entry2"));
 
-    entry = db->rootGroup()->findEntry(QString("group1/entry2"));
-    QVERIFY(entry != nullptr);
+    entry = db->rootGroup()->findEntryByPath(QString("group1/entry2"));
+    QVERIFY(entry);
     QCOMPARE(entry->title(), QString("entry2"));
 
-    entry = db->rootGroup()->findEntry(QString("/entry2"));
-    QVERIFY(entry == nullptr);
+    entry = db->rootGroup()->findEntryByPath(QString("/entry2"));
+    QVERIFY(!entry);
 
     // We also can find the entry with the leading slash.
-    entry = db->rootGroup()->findEntry(QString("/group1/entry2"));
-    QVERIFY(entry != nullptr);
+    entry = db->rootGroup()->findEntryByPath(QString("/group1/entry2"));
+    QVERIFY(entry);
     QCOMPARE(entry->title(), QString("entry2"));
 
     // Should also find the entry only by title.
-    entry = db->rootGroup()->findEntry(QString("entry2"));
-    QVERIFY(entry != nullptr);
+    entry = db->rootGroup()->findEntryByPath(QString("entry2"));
+    QVERIFY(entry);
     QCOMPARE(entry->title(), QString("entry2"));
 
-    entry = db->rootGroup()->findEntry(QString("invalid/path/to/entry2"));
-    QVERIFY(entry == nullptr);
+    entry = db->rootGroup()->findEntryByPath(QString("invalid/path/to/entry2"));
+    QVERIFY(!entry);
 
-    entry = db->rootGroup()->findEntry(QString("entry27"));
-    QVERIFY(entry == nullptr);
+    entry = db->rootGroup()->findEntryByPath(QString("entry27"));
+    QVERIFY(!entry);
 
     // A valid UUID that does not exist in this database.
-    entry = db->rootGroup()->findEntry(QString("febfb01ebcdf9dbd90a3f1579dc75281"));
-    QVERIFY(entry == nullptr);
+    entry = db->rootGroup()->findEntryByUuid(QUuid("febfb01ebcdf9dbd90a3f1579dc75281"));
+    QVERIFY(!entry);
 
     // An invalid UUID.
-    entry = db->rootGroup()->findEntry(QString("febfb01ebcdf9dbd90a3f1579dc"));
-    QVERIFY(entry == nullptr);
+    entry = db->rootGroup()->findEntryByUuid(QUuid("febfb01ebcdf9dbd90a3f1579dc"));
+    QVERIFY(!entry);
+
+    // Empty strings
+    entry = db->rootGroup()->findEntryByUuid({});
+    QVERIFY(!entry);
+
+    entry = db->rootGroup()->findEntryByPath({});
+    QVERIFY(!entry);
 }
 
 void TestGroup::testFindGroupByPath()
@@ -541,51 +568,51 @@ void TestGroup::testFindGroupByPath()
     Group* group;
 
     group = db->rootGroup()->findGroupByPath("/");
-    QVERIFY(group != nullptr);
+    QVERIFY(group);
     QCOMPARE(group->uuid(), db->rootGroup()->uuid());
 
     // We also accept it if the leading slash is missing.
     group = db->rootGroup()->findGroupByPath("");
-    QVERIFY(group != nullptr);
+    QVERIFY(group);
     QCOMPARE(group->uuid(), db->rootGroup()->uuid());
 
     group = db->rootGroup()->findGroupByPath("/group1/");
-    QVERIFY(group != nullptr);
+    QVERIFY(group);
     QCOMPARE(group->uuid(), group1->uuid());
 
     // We also accept it if the leading slash is missing.
     group = db->rootGroup()->findGroupByPath("group1/");
-    QVERIFY(group != nullptr);
+    QVERIFY(group);
     QCOMPARE(group->uuid(), group1->uuid());
 
     // Too many slashes at the end
     group = db->rootGroup()->findGroupByPath("group1//");
-    QVERIFY(group == nullptr);
+    QVERIFY(!group);
 
     // Missing a slash at the end.
     group = db->rootGroup()->findGroupByPath("/group1");
-    QVERIFY(group != nullptr);
+    QVERIFY(group);
     QCOMPARE(group->uuid(), group1->uuid());
 
     // Too many slashes at the start
     group = db->rootGroup()->findGroupByPath("//group1");
-    QVERIFY(group == nullptr);
+    QVERIFY(!group);
 
     group = db->rootGroup()->findGroupByPath("/group1/group2/");
-    QVERIFY(group != nullptr);
+    QVERIFY(group);
     QCOMPARE(group->uuid(), group2->uuid());
 
     // We also accept it if the leading slash is missing.
     group = db->rootGroup()->findGroupByPath("group1/group2/");
-    QVERIFY(group != nullptr);
+    QVERIFY(group);
     QCOMPARE(group->uuid(), group2->uuid());
 
     group = db->rootGroup()->findGroupByPath("group1/group2");
-    QVERIFY(group != nullptr);
+    QVERIFY(group);
     QCOMPARE(group->uuid(), group2->uuid());
 
     group = db->rootGroup()->findGroupByPath("invalid");
-    QVERIFY(group == nullptr);
+    QVERIFY(!group);
 }
 
 void TestGroup::testPrint()
@@ -601,7 +628,7 @@ void TestGroup::testPrint()
     Entry* entry1 = new Entry();
     entry1->setTitle(QString("entry1"));
     entry1->setGroup(db->rootGroup());
-    entry1->setUuid(Uuid::random());
+    entry1->setUuid(QUuid::createUuid());
 
     output = db->rootGroup()->print();
     QCOMPARE(output, QString("entry1\n"));
@@ -613,7 +640,7 @@ void TestGroup::testPrint()
 
     entry2->setTitle(QString("entry2"));
     entry2->setGroup(group1);
-    entry2->setUuid(Uuid::random());
+    entry2->setUuid(QUuid::createUuid());
 
     group1->setParent(db->rootGroup());
 
@@ -677,7 +704,7 @@ void TestGroup::testLocate()
     QVERIFY(results.contains("/entry1"));
 
     results = db->rootGroup()->locate("invalid");
-    QVERIFY(results.size() == 0);
+    QVERIFY(results.isEmpty());
 
     results = db->rootGroup()->locate("google");
     QVERIFY(results.size() == 1);
@@ -705,37 +732,111 @@ void TestGroup::testAddEntryWithPath()
     group2->setParent(group1);
 
     Entry* entry = db->rootGroup()->addEntryWithPath("entry1");
-    QVERIFY(entry != nullptr);
+    QVERIFY(entry);
     QVERIFY(!entry->uuid().isNull());
 
     entry = db->rootGroup()->addEntryWithPath("entry1");
-    QVERIFY(entry == nullptr);
+    QVERIFY(!entry);
 
     entry = db->rootGroup()->addEntryWithPath("/entry1");
-    QVERIFY(entry == nullptr);
+    QVERIFY(!entry);
 
     entry = db->rootGroup()->addEntryWithPath("entry2");
-    QVERIFY(entry != nullptr);
+    QVERIFY(entry);
     QVERIFY(entry->title() == "entry2");
     QVERIFY(!entry->uuid().isNull());
 
     entry = db->rootGroup()->addEntryWithPath("/entry3");
-    QVERIFY(entry != nullptr);
+    QVERIFY(entry);
     QVERIFY(entry->title() == "entry3");
     QVERIFY(!entry->uuid().isNull());
 
     entry = db->rootGroup()->addEntryWithPath("/group1/entry4");
-    QVERIFY(entry != nullptr);
+    QVERIFY(entry);
     QVERIFY(entry->title() == "entry4");
     QVERIFY(!entry->uuid().isNull());
 
     entry = db->rootGroup()->addEntryWithPath("/group1/group2/entry5");
-    QVERIFY(entry != nullptr);
+    QVERIFY(entry);
     QVERIFY(entry->title() == "entry5");
     QVERIFY(!entry->uuid().isNull());
 
     entry = db->rootGroup()->addEntryWithPath("/group1/invalid_group/entry6");
-    QVERIFY(entry == nullptr);
+    QVERIFY(!entry);
 
     delete db;
 }
+
+void TestGroup::testIsRecycled()
+{
+    Database* db = new Database();
+    db->rootGroup()->createRecycleBin();
+
+    Group* group1 = new Group();
+    group1->setName("group1");
+    group1->setParent(db->rootGroup());
+
+    Group* group2 = new Group();
+    group2->setName("group2");
+    group2->setParent(db->rootGroup());
+
+    Group* group3 = new Group();
+    group3->setName("group3");
+    group3->setParent(group2);
+
+    Group* group4 = new Group();
+    group4->setName("group4");
+    group4->setParent(db->rootGroup());
+
+    db->recycleGroup(group2);
+
+    QVERIFY(!group1->isRecycled());
+    QVERIFY(group2->isRecycled());
+    QVERIFY(group3->isRecycled());
+    QVERIFY(!group4->isRecycled());
+
+    db->recycleGroup(group4);
+    QVERIFY(group4->isRecycled());
+}
+
+void TestGroup::testCopyDataFrom()
+{
+    QScopedPointer<Group> group(new Group());
+    group->setName("TestGroup");
+
+    QScopedPointer<Group> group2(new Group());
+    group2->setName("TestGroup2");
+
+    QScopedPointer<Group> group3(new Group());
+    group3->setName("TestGroup3");
+    group3->customData()->set("testKey", "value");
+
+    QSignalSpy spyGroupModified(group.data(), SIGNAL(groupModified()));
+    QSignalSpy spyGroupDataChanged(group.data(), SIGNAL(groupDataChanged(Group*)));
+
+    group->copyDataFrom(group2.data());
+    QCOMPARE(spyGroupModified.count(), 1);
+    QCOMPARE(spyGroupDataChanged.count(), 1);
+
+    // if no change, no signals
+    spyGroupModified.clear();
+    spyGroupDataChanged.clear();
+    group->copyDataFrom(group2.data());
+    QCOMPARE(spyGroupModified.count(), 0);
+    QCOMPARE(spyGroupDataChanged.count(), 0);
+
+    // custom data change triggers a separate modified signal
+    spyGroupModified.clear();
+    spyGroupDataChanged.clear();
+    group->copyDataFrom(group3.data());
+    QCOMPARE(spyGroupDataChanged.count(), 1);
+    QCOMPARE(spyGroupModified.count(), 2);
+}
+
+void TestGroup::testEquals()
+{
+    QScopedPointer<Group> group(new Group());
+    group->setName("TestGroup");
+
+    QVERIFY(group->equals(group.data(), CompareItemDefault));
+}
